@@ -11,19 +11,19 @@ fi
 MAN="$1"
 SECTION="$2"
 
-if [ -d /usr/local/share/man ]
+if [ -d /usr/local/share/man/man1 ]
 then 
    MANTOP=/usr/local/share/man
 else
-   if [ -d /usr/share/man ]
+   if [ -d /usr/share/man/man1 ]
    then
       MANTOP=/usr/share/man
    else
-      if [ -d /usr/local/man ]
+      if [ -d /usr/local/man/man1 ]
       then
          MANTOP=/usr/local/man
       else
-	 if [ -d /usr/man ]
+	 if [ -d /usr/man/man1 ]
 	 then
 	    MANTOP=/usr/man
 	 else
