@@ -1,7 +1,7 @@
 /* 
  * httpsQueue.c
  * 
- * ufdbGuard is copyrighted (C) 2005-2010 by URLfilterDB with all rights reserved.
+ * ufdbGuard is copyrighted (C) 2005-2020 by URLfilterDB with all rights reserved.
  *
  * Parts of the ufdbGuard daemon are based on squidGuard.
  * This module is NOT based on squidGuard.
@@ -12,18 +12,22 @@
  * Since this queue is a FIFO we could use a tail queue. But we don't
  * like the malloc() overhead, so therefore we use a static array.
  *
- * RCS $Id: httpsQueue.c,v 1.15 2010/10/07 15:08:08 root Exp root $
+ * RCS $Id: httpsQueue.c,v 1.24 2020/06/17 07:14:43 root Exp root $
  */
 
 #include "ufdb.h"
 #include "ufdblib.h"
+#include "ufdblocks.h"
 #include "httpsQueue.h"
 
 #include <unistd.h>
 #include <string.h>
-#include <pthread.h>
 
-#define UFDB_HTTPS_QUEUE_SIZE	 256
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+#define UFDB_HTTPS_QUEUE_SIZE	 1500
 
 static struct {
    int    portnumber;
@@ -33,7 +37,7 @@ static struct {
 static volatile int    ihead = 0;                    /* array index for the head */
 static volatile int    itail = 0;                    /* array index for the tail */
 static volatile int    _ufdb_httpsq_n_queued = 0;
-static pthread_mutex_t httpsq_lock = UFDB_STATIC_MUTEX_INIT;
+static pthread_mutex_t httpsq_lock = PTHREAD_MUTEX_INITIALIZER;
 static pthread_cond_t  empty = PTHREAD_COND_INITIALIZER;
 
 #if 0
@@ -45,16 +49,18 @@ static pthread_cond_t  empty = PTHREAD_COND_INITIALIZER;
  * enqueue a hostname:portnumber for future https tunnel detection.
  */
 int ufdbHttpsQueueRequest( 
-   char * hostname, 
-   int    portnumber )
+   const char * hostname, 
+   int          portnumber )
 {
    int    i;
-   int    rv;
+   int    rv;		/* must ignore warning about variable set but not used */
 
    rv = pthread_mutex_lock( &httpsq_lock );
-#if defined(UFDB_DEBUG) || defined(UFDB_DEBUG_QUEUE)
    if (rv != 0)
+#if defined(UFDB_DEBUG) || defined(UFDB_DEBUG_QUEUE)
       ufdbLogError( "ufdbHttpsQueueRequest: mutex_lock failed rv=%d *****", rv );
+#else
+      { ; }
 #endif
 
    /*
@@ -66,9 +72,11 @@ int ufdbHttpsQueueRequest(
           q[i].portnumber == portnumber)
       {
 	 rv = pthread_mutex_unlock( &httpsq_lock );
-#ifdef UFDB_DEBUG
 	 if (rv != 0)
+#ifdef UFDB_DEBUG
 	    ufdbLogError( "ufdbHttpsQueueRequest: mutex_unlock failed rv=%d *****", rv );
+#else
+	    { ; }
 #endif
 #if defined(UFDB_DEBUG) || defined(UFDB_DEBUG_QUEUE)
          ufdbLogMessage( "ufdbHttpsQueueRequest: duplicate at %d: %s:%d", i, hostname, portnumber );
@@ -96,9 +104,11 @@ int ufdbHttpsQueueRequest(
        * leave the critical section
        */
       rv = pthread_mutex_unlock( &httpsq_lock );
-#ifdef UFDB_DEBUG
       if (rv != 0)
+#ifdef UFDB_DEBUG
 	 ufdbLogError( "ufdbHttpsQueueRequest: mutex_unlock failed rv=%d *****", rv );
+#else
+         { ; }
 #endif
 
       /*
@@ -113,13 +123,15 @@ int ufdbHttpsQueueRequest(
    else    
    {
       /* queue is full; just drop the request */
-      ufdbLogError( "HTTPS/SSL security detection queue is full. %s:%d is not checked now", 
-                    hostname, portnumber );
+      ufdbLogMessage( "Warning: HTTPS/SSL security detection queue is full. %s:%d is not checked now", 
+                      hostname, portnumber );
 
       rv = pthread_mutex_unlock( &httpsq_lock );
-#ifdef UFDB_DEBUG
       if (rv != 0)
+#ifdef UFDB_DEBUG
 	 ufdbLogError( "ufdbHttpsQueueRequest: (full) mutex_unlock failed rv=%d *****", rv );
+#else
+         { ; }
 #endif
 
 #ifdef _POSIX_PRIORITY_SCHEDULING
@@ -138,13 +150,15 @@ int ufdbHttpsQueueRequest(
  */
 void ufdbGetHttpsRequest( char * hostname, int * portnumber )
 {
-   int rv;
+   int rv;		/* must ignore warning about variable set but not used */
 
 allover:
    rv = pthread_mutex_lock( &httpsq_lock );
-#ifdef UFDB_DEBUG
    if (rv != 0)
+#ifdef UFDB_DEBUG
       ufdbLogError( "ufdbGetHttpsRequest: mutex_lock failed with rv=%d *****", rv );
+#else
+      { ; }
 #endif
 
 #if defined(UFDB_DEBUG) || defined(UFDB_DEBUG_QUEUE)
@@ -183,9 +197,11 @@ allover:
 	  * usleep() to try to prevent a race with old pthread libraries.
           */
          rv = pthread_mutex_unlock( &httpsq_lock );
-#ifdef UFDB_DEBUG
 	 if (rv != 0)
+#ifdef UFDB_DEBUG
 	    ufdbLogError( "ufdbGetHttpsRequest: mutex_unlock failed with rv=%d *****", rv );
+#else
+            { ; }
 #endif
 	 usleep( ((unsigned long) pthread_self()) % 1025 + 201 );
 	 goto allover;
@@ -203,3 +219,8 @@ int UFDBhttpsVerificationQueued( void )
    return _ufdb_httpsq_n_queued;
 }
 
+
+#ifdef __cplusplus
+}
+#endif
+
