@@ -5,6 +5,10 @@
 #include "ufdblib.h"
 #include <stdio.h>
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 int main()
 {
    int i;
@@ -38,7 +42,7 @@ int main()
 
    printf( "#IP: %lu (9)\n", UFDBgetNumberOfRegisteredIPs() );
 
-   UFDBregisterCountedIP( "10.222.83.107" );
+   UFDBregisterCountedIP( "10.63.83.107" );
    UFDBregisterCountedIP( "10.222.83.108" );
    UFDBregisterCountedIP( "254.252.231.244" );
    UFDBregisterCountedIP( "254.253.231.244" );
@@ -49,11 +53,12 @@ int main()
    UFDBregisterCountedIP( "pc02.neverseenbeforeextremelyverylonginternalmydomain01.net" );
    UFDBregisterCountedIP( "pc03.neverseenbeforeextremelyverylonginternalmydomain01.net" );
 
+#if 1
    /* time iptst takes 0.40 sec for 10 million+ calls on old Xeon 2.8 GHz 512 KB cache */
    for (i = 10000000; i>0; i--)
-      UFDBregisterCountedIP( "10.222.83.107" );
-
-   printf( "#IP: %lu (16)\n", UFDBgetNumberOfRegisteredIPs() );
+      UFDBregisterCountedIP( "10.22.183.177" );
+#endif
+   printf( "#IP: %lu (17)\n", UFDBgetNumberOfRegisteredIPs() );
 
    UFDBregisterCountedIP( "10.0.0.14" );
    UFDBregisterCountedIP( "10.0.0.15" );
@@ -86,10 +91,27 @@ int main()
    UFDBregisterCountedIP( "10.0.0.42" );
    UFDBregisterCountedIP( "10.0.0.43" );
    UFDBregisterCountedIP( "10.0.0.44" );
-   UFDBregisterCountedIP( "10.0.0.45" );
 
-   printf( "#IP: %lu (48)\n", UFDBgetNumberOfRegisteredIPs() );
+   UFDBregisterCountedIP( "10.0.0.45" );
+   printf( "#IP: %lu (49)\n", UFDBgetNumberOfRegisteredIPs() );
+
+   UFDBregisterCountedIP( "11.0.0.14" );
+
+   UFDBregisterCountedIP( "11.0.0.15" );
+
+   UFDBregisterCountedIP( "11.1.1.14" );
+   printf( "#IP: %lu (50)\n", UFDBgetNumberOfRegisteredIPs() );
+
+   UFDBregisterCountedIP( "11.255.255.14" );
+   UFDBregisterCountedIP( "11.255.255.254" );
+   UFDBregisterCountedIP( "11.255.255.255" );
+
+   printf( "#IP: %lu (53)\n", UFDBgetNumberOfRegisteredIPs() );
 
    return 0;
 }
 
+
+#ifdef __cplusplus
+}
+#endif
