@@ -28,6 +28,10 @@
 #include <signal.h>
 #include <pthread.h>
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 extern char **environ;
 
 
@@ -70,3 +74,7 @@ int pthread_cond_wait(pthread_cond_t *cond, pthread_mutex_t *mutex)
    return 0;
 }
 
+
+#ifdef __cplusplus
+}
+#endif
