@@ -1,7 +1,7 @@
 /*
  * ufdbguard,c - URLfilterDB
  *
- * ufdbGuard is copyrighted (C) 2005-2009 by URLfilterDB with all rights reserved.
+ * ufdbGuard is copyrighted (C) 2005-2020 by URLfilterDB with all rights reserved.
  *
  * Parts of the ufdbGuard daemon are based on squidGuard.
  * squidGuard is copyrighted (C) 1998 by
@@ -21,6 +21,7 @@
 #include "ufdb.h"
 #include "sg.h"
 #include "ufdblib.h"
+#include "ufdbdb.h"
 #include "ufdbchkport.h"
 
 #include <string.h>
@@ -30,6 +31,10 @@
 #include <signal.h>
 #include <sys/time.h>
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /* use linebuffer output: flush() is not needed any more */
 #if 0
 #define FLUSH(f)   fflush(f)
@@ -42,7 +47,6 @@ static int parseOnly = 0;
 static int noRedirects = 0;
 
 
-
 static void usage( void )
 {
   fprintf( stderr, 
@@ -64,6 +68,29 @@ static void usage( void )
 }
 
 
+void sgHandlerSigHUP( int s )
+{
+   ufdbGV.sig_hup = s;
+}
+
+
+void sgSignalHandler( int s )
+{
+   static int num = 0;
+
+   if (ufdbGV.sig_other == s)
+   {
+      num++;
+      if (num >= 3)
+	 abort();
+   }
+   else
+      num = 0;
+
+   ufdbGV.sig_other = s;
+}
+
+
 int ufdbguard_main( int argc, char ** argv, char ** envp )
 {
   int    ch;
@@ -82,25 +109,25 @@ int ufdbguard_main( int argc, char ** argv, char ** envp )
   char   buf[UFDB_MAX_URL_LENGTH];
   char   tmp[UFDB_MAX_URL_LENGTH];
 
-  strcpy( progname, "ufdbGuard" );
+  strcpy( ufdbGV.progname, "ufdbguard" );
 
   UFDBtimerInit( &timer );
   admin = UFDBallocThreadAdmin();
 
   /* instead of using fflush(), set linebuffer mode for stdout */
-  vbuf = ufdbMalloc( 4096 );
+  vbuf = (char *) ufdbMalloc( 4096 );
   (void) setvbuf( stdout, vbuf, _IOLBF, 4096 );
 
-  gettimeofday( &start_time, NULL );
-  globalPid = getpid();
+  UFDBgettimeofday( &start_time, NULL );
+  ufdbGV.pid = getpid();
   while ((ch = getopt(argc, argv, "Phdrt:c:vCu")) > 0)
   {
     switch (ch) {
     case 'd':
-      UFDBglobalDebug = 1;
+      ufdbGV.debug = 1;
       break;
     case 'r':
-      globalDebugRedirect = 1;
+      ufdbGV.debugRedirect = 1;
       break;
     case 'c':
       configFile = optarg;
@@ -111,7 +138,7 @@ int ufdbguard_main( int argc, char ** argv, char ** envp )
       exit( 1 );
       break;
     case 'v':
-      fprintf( stderr, "ufdbGuard: %s\n", VERSION );
+      fprintf( stderr, "ufdbGuard: %s\n", UFDB_VERSION );
       exit( 0 );
       break;
     case 't':
@@ -124,12 +151,12 @@ int ufdbguard_main( int argc, char ** argv, char ** envp )
 	exit(0);
       }
       niso( t, niso_str );
-      ufdbLogError( "ufdbGuard emulating date %s", niso_str );
+      ufdbLogMessage( "ufdbGuard emulating date %s", niso_str );
       globalDebugTimeDelta = t - start_time.tv_sec;
       start_time.tv_sec = start_time.tv_sec + globalDebugTimeDelta;
       break;
     case 'T':
-      ufdbLogError( "-T option found.  Going into test mode." );
+      ufdbLogMessage( "-T option found.  Going into test mode." );
       noRedirects = 1;		/* Test mode means noRedirects */
       break;
     case 'P':			/* undocumented -P option for development purposes */
@@ -144,12 +171,15 @@ int ufdbguard_main( int argc, char ** argv, char ** envp )
 
   globalArgv = argv;
   globalEnvp = envp;
-  ufdbSetGlobalErrorLogFile();
-  sgReadConfig( configFile );
-  /* ufdbSetGlobalErrorLogFile(); */
-  ufdbLogMessage( "ufdbGuard " VERSION " started at %d.%03d", 
-                  start_time.tv_sec, start_time.tv_usec/1000 );
-  if (UFDBglobalDebug)
+  ufdbSetGlobalErrorLogFile( NULL, NULL, 0 );
+  if (ufdbReadConfig( configFile ) == 0)
+  {
+     ufdbLogFatalError( "missing configuration file" );
+     exit( 3 );
+  }
+  ufdbLogMessage( "ufdbGuard " UFDB_VERSION " started at %ld.%03ld", 
+                  (long) start_time.tv_sec, (long) start_time.tv_usec/1000 );
+  if (ufdbGV.debug)
      ufdbLogMessage( "debug output is enabled." );
 
   ufdbSetThreadCPUaffinity( 0 );
@@ -157,15 +187,14 @@ int ufdbguard_main( int argc, char ** argv, char ** envp )
 
   if (globalUpdate || globalCreateDb != NULL)
   {
-    ufdbLogError( "db update done" );
-    gettimeofday( &stop_time, NULL );
+    ufdbLogMessage( "db update done" );
+    UFDBgettimeofday( &stop_time, NULL );
     stop_time.tv_sec = stop_time.tv_sec + globalDebugTimeDelta;
-    ufdbLogError( "ufdbGuard stopped at %d.%03d",
-	          stop_time.tv_sec, stop_time.tv_usec/1000 );
+    ufdbLogMessage( "ufdbGuard stopped at %ld.%03ld",
+	            (long) stop_time.tv_sec, (long) stop_time.tv_usec/1000 );
     exit( 0 );
   }
 
-  sgTimeElementSortEvents();
   ufdbHandleAlarmForTimeEvents( UFDB_PARAM_INIT );
 
   ufdbSetSignalHandler( SIGHUP,  sgHandlerSigHUP );
@@ -181,10 +210,10 @@ int ufdbguard_main( int argc, char ** argv, char ** envp )
 
   UFDBlogConfig();
 
-  gettimeofday( &ready_time, NULL );
+  UFDBgettimeofday( &ready_time, NULL );
   ready_time.tv_sec = ready_time.tv_sec + globalDebugTimeDelta;
-  ufdbLogError( "ufdbGuard ready for requests at %d.%03d",
-	        ready_time.tv_sec, ready_time.tv_usec/1000 );
+  ufdbLogMessage( "ufdbGuard ready for requests at %ld.%03ld",
+	        (long) ready_time.tv_sec, (long) ready_time.tv_usec/1000 );
 
 #if 1
   UFDBtimerStop( &timer );
@@ -192,7 +221,7 @@ int ufdbguard_main( int argc, char ** argv, char ** envp )
   UFDBtimerInit( &timer );
 #endif
 
-  UFDBglobalReconfig = 0;
+  ufdbGV.reconfig = UFDB_RECONFIGR_NONE;
 
   while (1)
   {
@@ -201,14 +230,14 @@ int ufdbguard_main( int argc, char ** argv, char ** envp )
       buf[UFDB_MAX_URL_LENGTH-1] = '\0';   		/* force null termination */
       if (sig_hup)
       {
-	sgReloadConfig();
+	ufdbReloadConfig();
 	sig_hup = 0;
       }
       if (sig_other)
         break;
       numreq++;
 
-      if (UFDBglobalFatalError || failsafe_mode) {
+      if (ufdbGV.fatalError || failsafe_mode) {
 	putchar( '\n' );		/* OK */
 	FLUSH( stdout );
 	continue;
@@ -225,28 +254,27 @@ int ufdbguard_main( int argc, char ** argv, char ** envp )
       }
       else
       {
-	for (src = Source; ; )
+	for (src = (struct Source *) SourceList; ; )
 	{
 	  int access;
 
-	  strcpy( tmp, squidInfo.src );		/* TODO: find out if strcpy() is necessary... */
-	  src = sgFindSource( src, tmp, &squidInfo );
-	  acl = sgAclCheckSource( src );
+	  src = UFDBfindSource( ufdbGV.reconfig, src, &squidInfo );
+	  acl = UFDBfindACLbySource( ufdbGV.reconfig, src, &squidInfo );
 	  if (parseOnly)
 	  {
 	    putchar( '\n' );		/* OK */
 	    break;
 	  }
 	  redirect = tmp;
-	  access = UFDBCheckACLaccess( src, acl, &squidInfo, redirect );
+	  access = UFDBdecideAccessBasedOnURL( 0, acl, &squidInfo, redirect );
 	  if (access == UFDB_ACL_ACCESS_DUNNO  ||  access == UFDB_ACL_ACCESS_ALLOW)
 	  {
 	    /* src not found */
 	    if (src == NULL || src->cont_search == 0) 
 	    {
-	       if (UFDBglobalLogAllRequests)
+	       if (ufdbGV.logAllRequests)
 	       {
-		  char * category;
+		  const char * category;
 		  if (squidInfo.aclpass == NULL)
 		     category = "unknown";
 		  else if (squidInfo.aclpass->name == NULL)
@@ -260,8 +288,8 @@ int ufdbguard_main( int argc, char ** argv, char ** envp )
 		     squidInfo.ident[1] = '\0';
 		  }
 		  ufdbLogMessage( "PASS  %-8s %-15s %-10s %-9s %s",
-				  squidInfo.ident, squidInfo.src, acl->name, category,
-				  squidInfo.url2display );
+				  squidInfo.ident, squidInfo.srcIP, acl->name, category,
+				  squidInfo.url2log );
 	       }
 
 	      putchar( '\n' );		/* OK */
@@ -274,9 +302,9 @@ int ufdbguard_main( int argc, char ** argv, char ** envp )
 	      }
               else
               {
-		  if (UFDBglobalLogAllRequests)
+		  if (ufdbGV.logAllRequests)
 		  {
-		     char * category;
+		     const char * category;
 		     if (squidInfo.aclpass == NULL)
 			category = "unknown";
 		     else if (squidInfo.aclpass->name == NULL)
@@ -290,8 +318,8 @@ int ufdbguard_main( int argc, char ** argv, char ** envp )
 			squidInfo.ident[1] = '\0';
 		     }
 		    ufdbLogMessage( "PASS  %-8s %-15s %-10s %-9s %s",
-				    squidInfo.ident, squidInfo.src, acl->name, category,
-				    squidInfo.url2display );
+				    squidInfo.ident, squidInfo.srcIP, acl->name, category,
+				    squidInfo.url2log );
 	        }
 
 		putchar( '\n' );  	/* OK */
@@ -311,9 +339,9 @@ int ufdbguard_main( int argc, char ** argv, char ** envp )
 	      squidInfo.ident[1] = '\0';
 	    }
 
-            if (globalDebugRedirect)
+            if (ufdbGV.debugRedirect)
 	    {
-	       ufdbLogMessage( "REDIRECT %s %s/%s %s %s %s\n", redirect, squidInfo.src,
+	       ufdbLogMessage( "REDIRECT %s %s/%s %s %s %s\n", redirect, squidInfo.srcIP,
 			       squidInfo.srcDomain, squidInfo.ident, "GET", squidInfo.urlgroup );
 	    }
 
@@ -322,7 +350,7 @@ int ufdbguard_main( int argc, char ** argv, char ** envp )
 	    else
 	    {
 	       char urlgrp[64];
-	       char * category;
+	       const char * category;
 
 	       if (squidInfo.aclpass == NULL)
 		  category = "unknown";
@@ -339,14 +367,14 @@ int ufdbguard_main( int argc, char ** argv, char ** envp )
 	          strncpy( &urlgrp[1], squidInfo.urlgroup, 62 );
 		  urlgrp[63] = '\0';
 	       }
-	       printf( "%s %s/%s %s %s%s\n", redirect, squidInfo.src,
+	       printf( "%s %s/%s %s %s%s\n", redirect, squidInfo.srcIP,
 			squidInfo.srcDomain, squidInfo.ident, "GET", urlgrp );
 
-	       if (UFDBglobalLogAllRequests)
+	       if (ufdbGV.logAllRequests)
 	       {
 		    ufdbLogMessage( "BLOCK %-8s %-15s %-10s %-9s %s",
-				    squidInfo.ident, squidInfo.src, acl->name, category,
-				    squidInfo.url2display );
+				    squidInfo.ident, squidInfo.srcIP, acl->name, category,
+				    squidInfo.url2log );
 	       }
 	    }
 	    break;
@@ -359,7 +387,7 @@ int ufdbguard_main( int argc, char ** argv, char ** envp )
       if (sig_hup)
       {
         sig_hup = 0;
-        sgReloadConfig();
+        ufdbReloadConfig();
       }
       if (sig_other)
         break;
@@ -369,18 +397,18 @@ int ufdbguard_main( int argc, char ** argv, char ** envp )
 
     if (sig_other)
     {
-      ufdbLogError( "signal %d received. exiting...", sig_other );
+      ufdbLogMessage( "signal %d received. exiting...", sig_other );
       exit( 2 );
     }
 
 #if 1
     UFDBtimerStop( &timer );
-    ufdbLogError( "processed %ld requests", numreq );
+    ufdbLogMessage( "processed %ld requests", numreq );
     UFDBtimerPrint( &timer, "time used to handle requests" );
 #endif
 
     if (feof(stdin))
-      ufdbLogError( "EOF on input stream" );
+      ufdbLogMessage( "EOF on input stream" );
 
     fclose( stdin );
     fclose( stdout );
@@ -394,18 +422,18 @@ int ufdbguard_main( int argc, char ** argv, char ** envp )
 #if HAVE_SIGNAL
     if (errno != EINTR) 
     {
-      gettimeofday( &stop_time, NULL );
+      UFDBgettimeofday( &stop_time, NULL );
       stop_time.tv_sec = stop_time.tv_sec + globalDebugTimeDelta;
-      ufdbLogError( "ufdbGuard stopped at %d.%03d",
+      ufdbLogMessage( "ufdbGuard stopped at %d.%03d",
 	            stop_time.tv_sec, stop_time.tv_usec/1000 );
       exit(2);
     }
 #endif
 #else 
-    gettimeofday( &stop_time, NULL );
+    UFDBgettimeofday( &stop_time, NULL );
     stop_time.tv_sec = stop_time.tv_sec + globalDebugTimeDelta;
-    ufdbLogError( "ufdbGuard stopped at %d.%03d",
-	          stop_time.tv_sec, stop_time.tv_usec/1000 );
+    ufdbLogMessage( "ufdbGuard stopped at %ld.%03ld",
+	          (long) stop_time.tv_sec, (long) stop_time.tv_usec/1000 );
     exit(0);
 #endif
   } /* while (1) */
@@ -415,17 +443,103 @@ int ufdbguard_main( int argc, char ** argv, char ** envp )
 }
 
 
-void sgReloadConfig( void )
+void ufdbReloadConfig( void )
 {
   sig_hup = 0;
-  ufdbLogError( "got sigHUP reload config" );
+  ufdbLogMessage( "got sigHUP reload config" );
 
   /* closing global logfile */
-  fclose( globalErrorLog );
-  globalErrorLog = NULL;
+  fclose( ufdbGlobalLogfile );
+  ufdbGlobalLogfile = NULL;
 
   execve( *globalArgv, globalArgv, globalEnvp );
   fprintf( stderr, "error execve: %d\n", errno );
 }
 
 
+static char * my_fast_fgets_nonl( char * s, int size, FILE * stream )
+{
+   char * buf;
+   int    ch;
+
+   buf = s;
+   while ((ch = getc_unlocked(stream)) != EOF  &&  --size > 0)
+   {
+      if (ch == '\n'  ||  ch == '\r')
+         break;
+      *buf++ = ch;
+   }
+   *buf = '\0';
+   if (ch == EOF  &&  buf == s)
+      return NULL;
+   return s;
+}
+
+
+void execute_pstack( char * reason )
+{
+   if (strcmp( UFDB_GDB_PATH, "no" ) == 0)
+   {
+      ufdbLogMessage( "ufdb-pstack is not called since the gdb package was not installed." );
+      ufdbLogMessage( "It is highly recommended to install gdb to produce a crash report.  *****" );
+      ufdbLogMessage( "Crash reports are essential to be able to resolve bugs quickly." );
+   }
+   else
+   {
+      FILE * fp;
+      char   cmd[512];
+      char   buffer[2048];
+
+#if 0
+      ufdbSetSignalHandler( SIGALRM, interruptPstack );
+      alarm( 6 );
+#endif
+
+      sprintf( cmd, DEFAULT_BINDIR "/ufdb-pstack %d %s 2>&1", getpid(), reason );
+
+      /* gdb sometimes needs root permissions, so the popen() is done as root */
+      UFDBraisePrivileges( ufdbGV.userName, "open pipe for ufdb-pstack" );
+      errno = 0;
+      fp = popen( cmd, "r" );
+      if (fp == NULL)
+         ufdbLogError( "could not open pipe and execute \"%s\": %s", cmd, strerror(errno) );
+
+      while (my_fast_fgets_nonl( buffer, 2040, fp ) != NULL)
+      {
+         ufdbLogMessage( "pstack  %s", buffer );
+      }
+      pclose( fp );
+      ufdbLogMessage( "pstack  END" );
+      UFDBdropPrivileges( ufdbGV.userName );
+   }
+}
+
+
+void BadSignalCaught( int signum )	
+{
+   char      me[48];
+   
+   /* find out which thread has a signal */
+   sprintf( me, "pid %d", (int) getpid() );
+   ufdbLogMessage( "%s caught bad signal %d.", me, signum );
+
+   execute_pstack( me );
+}
+
+
+void __fortify_fail( const char * msg )
+{
+   ufdbGV.reconfig = UFDB_RECONFIGR_FORTIFY;
+   ufdbGV.terminating = 1;
+   (void) alarm( 0 );
+
+   ufdbLogMessage( "fortified code failed: %s.  simulating bad signal to call ufdb-pstack...", msg );
+
+   BadSignalCaught( SIGQUIT );
+   _exit( 4 );
+}
+
+
+#ifdef __cplusplus
+}
+#endif
