@@ -1,7 +1,7 @@
 /*
  * ufdbhttpd.c - URLfilterDB
  *
- * Copyrighted (C) 2007-2010 by URLfilterDB with all rights reserved.
+ * Copyrighted (C) 2007-2023 by URLfilterDB B.V. with all rights reserved.
  *
  * Parts of the ufdbGuard daemon are based on squidGuard.
  * This module is NOT based on squidGuard.
@@ -29,6 +29,10 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 
 int    runAsDaemon = 1;
 
@@ -36,7 +40,7 @@ char   interface[256] = "all";
 char   imagesDirectory[256] = "";
 int    portnumber = 0;
 
-char * globalPidFile = UFDBHTTPD_PID_FILE;
+char * globalHttpdPidFile = (char *) UFDBHTTPD_PID_FILE;
 int    UFDBforceLogRotation = 0;
 static char   UFDBlogFilename[1024];
 static size_t logfilesize = 0;
@@ -45,6 +49,10 @@ static size_t logfilesize = 0;
 static void usage( void )
 {
    fprintf( stderr, "usage: ufdbhttpd [-d] [-U <user>] -p <port> -I <images-directory> -l <log-directory> [-i <interface>]\n" );
+
+   fprintf( stderr, "The ufdbGuard software suite is free and Open Source Software.\n" );
+   fprintf( stderr, "Copyright (C) 2005-2023 by URLfilterDB B.V. and others.\n" );
+
    exit( 1 );
 }
 
@@ -56,8 +64,8 @@ static void usage( void )
 static void RotateLogfile( char * filename )
 {
    int         i;
-   char        oldfile[1024];
-   char        newfile[1024];
+   char        oldfile[1024+12];
+   char        newfile[1024+12];
 
    /* rotate the log file:
     * file.log.7  ->  file.log.8
@@ -72,18 +80,20 @@ static void RotateLogfile( char * filename )
 
    for (i = 8; i > 1; i--)
    {
-      sprintf( newfile, "%s.%d", filename, i );
-      sprintf( oldfile, "%s.%d", filename, i-1 );
+      snprintf( newfile, sizeof(newfile), "%s.%d", filename, i );
+      snprintf( oldfile, sizeof(oldfile), "%s.%d", filename, i-1 );
       rename( oldfile, newfile );
    }
+   sync();		/* sync may resolve some borderline disk space issues */
 
-   sprintf( newfile, "%s.%d", filename, 1 );
+   snprintf( newfile, sizeof(newfile), "%s.%d", filename, 1 );
    rename( filename, newfile );
 }
 
 
 void ufdbGlobalSetLogging( int logging )
 {
+   if (logging) { ; }           // prevent compiler warning
 }
 
 
@@ -93,56 +103,64 @@ void UFDBrotateLogfile( void )
 }
 
 
-void ufdbSetGlobalErrorLogFile( void )
+void ufdbSetGlobalErrorLogFile( char * logdir, char * basename, int mutex_is_used )
 {
    struct stat s;
 
-   if (globalErrorLog != NULL)
+   if (mutex_is_used) { ; }     // prevent compiler warning
+
+   if (ufdbGlobalLogfile != NULL)
    {
-      fclose( globalErrorLog );
-      globalErrorLog = NULL;
+      fclose( ufdbGlobalLogfile );
+      ufdbGlobalLogfile = NULL;
    }
 
-   if (UFDBglobalLogDir == NULL)
-      strcpy( UFDBlogFilename, DEFAULT_LOGDIR );
+   if (logdir == NULL)
+   {
+      if (ufdbGV.logDirectory[0] == '\0')
+         strcpy( UFDBlogFilename, DEFAULT_LOGDIR );
+      else
+         strcpy( UFDBlogFilename, ufdbGV.logDirectory );
+   }
    else
-      strcpy( UFDBlogFilename, UFDBglobalLogDir );
+      strcpy( UFDBlogFilename, logdir );
+
    strcat( UFDBlogFilename, "/" );
 
-   if (progname != NULL  &&  progname[0] != '\0')
+   if (basename == NULL  ||  basename[0] == '\0')
+      strcat( UFDBlogFilename, DEFAULT_LOGFILE );
+   else
    {
-      strcat( UFDBlogFilename, progname );
+      strcat( UFDBlogFilename, basename );
       strcat( UFDBlogFilename, ".log" );
    }
-   else
-      strcat( UFDBlogFilename, DEFAULT_LOGFILE );
 
    if (stat(UFDBlogFilename,&s) == 0)
    {
-      if (s.st_size > UFDBglobalMaxLogfileSize)
+      if (s.st_size > (off_t) ufdbGV.maxLogfileSize)
 	 RotateLogfile( UFDBlogFilename );
    }
 
-   globalErrorLog = fopen( UFDBlogFilename, "a" );
-   if (globalErrorLog == NULL)
+   ufdbGlobalLogfile = fopen( UFDBlogFilename, "a" );
+   if (ufdbGlobalLogfile == NULL)
    {
       ufdbLogError( "%s: can't write to logfile %s; %s (uid=%d,euid=%d)", 
-      	 	    progname, UFDBlogFilename, strerror(errno), getuid(), geteuid() );
+      	 	    ufdbGV.progname, UFDBlogFilename, strerror(errno), getuid(), geteuid() );
       /*
        * We *want* a logfile, try an other directory...
        */
       strcpy( UFDBlogFilename, "/tmp/" );
-      strcat( UFDBlogFilename, progname );
+      strcat( UFDBlogFilename, ufdbGV.progname );
       strcat( UFDBlogFilename, ".log" );
-      globalErrorLog = fopen( UFDBlogFilename, "a" );
-      if (globalErrorLog == NULL)
-	 ufdbLogError( "%s: can't write to logfile %s; %s", progname, UFDBlogFilename, strerror(errno) );
+      ufdbGlobalLogfile = fopen( UFDBlogFilename, "a" );
+      if (ufdbGlobalLogfile == NULL)
+	 ufdbLogError( "%s: can't write to logfile %s; %s", ufdbGV.progname, UFDBlogFilename, strerror(errno) );
    }
 
-   if (globalErrorLog == NULL)
+   if (ufdbGlobalLogfile == NULL)
       logfilesize = 0;
    else
-      logfilesize = ftell( globalErrorLog );
+      logfilesize = ftell( ufdbGlobalLogfile );
 }
 
 
@@ -154,13 +172,13 @@ static void niso(
   struct tm lc;
 
   if (t == 0)
-    tp = time(NULL) + globalDebugTimeDelta;
+    tp = time(NULL);
   else
     tp = t;
   localtime_r( &tp, &lc );
-  sprintf( buf, "%04d-%02d-%02d %02d:%02d:%02d", 
-          lc.tm_year + 1900, lc.tm_mon + 1,
-	  lc.tm_mday, lc.tm_hour, lc.tm_min, lc.tm_sec );
+  snprintf( buf, 80, "%04d-%02d-%02d %02d:%02d:%02d", 
+            lc.tm_year + 1900, lc.tm_mon + 1,
+	    lc.tm_mday, lc.tm_hour, lc.tm_min, lc.tm_sec );
 }
 
 
@@ -170,7 +188,7 @@ static void ufdbLog(
 {
    char *    nl;
    int       multiline;
-   char      date[22];
+   char      date[80];
    char      logmsg[UFDB_MAX_URL_LENGTH+128];
    int       logmsglen;
 
@@ -182,23 +200,23 @@ static void ufdbLog(
    while ((nl = strchr( msg, '\n' )) != NULL)
    {
       *nl = '\0';
-      logmsglen += sprintf( logmsg+logmsglen, "%s [%d] %s%s\n", date, globalPid, multiline?"   ":"", msg );
+      logmsglen += sprintf( logmsg+logmsglen, "%s [%d] %s%s\n", date, ufdbGV.pid, multiline?"   ":"", msg );
       msg = nl + 1;
       multiline = 1;
    }
-   logmsglen += sprintf( logmsg+logmsglen, "%s [%d] %s%s\n", date, globalPid, multiline?"   ":"", msg );
+   logmsglen += sprintf( logmsg+logmsglen, "%s [%d] %s%s\n", date, ufdbGV.pid, multiline?"   ":"", msg );
 
    if (UFDBforceLogRotation)
    {
       UFDBforceLogRotation = 0;
-      if (globalErrorLog != NULL)  
+      if (ufdbGlobalLogfile != NULL)  
       {
 	 RotateLogfile( UFDBlogFilename );
-	 ufdbSetGlobalErrorLogFile();
+	 ufdbSetGlobalErrorLogFile( NULL, NULL, 1 );
       }
    }
 
-   if (globalErrorLog == NULL) 
+   if (ufdbGlobalLogfile == NULL) 
    {
      fputs( logmsg, stderr );
      fflush( stderr );
@@ -206,23 +224,23 @@ static void ufdbLog(
    else
    {
       if (logfilesize == 0)
-	 logfilesize = ftell( globalErrorLog );
+	 logfilesize = ftell( ufdbGlobalLogfile );
 
       /* write is *much* faster than fputs */
-      if (write( fileno(globalErrorLog), logmsg, (size_t) logmsglen ) > 0)
+      if (write( fileno(ufdbGlobalLogfile), logmsg, (size_t) logmsglen ) > 0)
 	 logfilesize += logmsglen;
 
-      if (logfilesize > UFDBglobalMaxLogfileSize)
+      if (logfilesize > (size_t) ufdbGV.maxLogfileSize)
       {
 	 RotateLogfile( UFDBlogFilename );
-	 ufdbSetGlobalErrorLogFile();
+	 ufdbSetGlobalErrorLogFile( NULL, NULL, 1 );
 	 logfilesize = 0;
       }
    }
 }
 
 
-void ufdbLogError( char * format, ... )
+void ufdbLogError( const char * format, ... )
 {
    va_list ap;
    char    msg[UFDB_MAX_URL_LENGTH];
@@ -236,7 +254,7 @@ void ufdbLogError( char * format, ... )
 }
 
 
-void ufdbLogMessage( char * format, ... )
+void ufdbLogMessage( const char * format, ... )
 {
    va_list ap;
    char    msg[UFDB_MAX_URL_LENGTH];
@@ -250,11 +268,11 @@ void ufdbLogMessage( char * format, ... )
 }
 
 
-void ufdbLogFatalError( char * format, ... )
+void ufdbLogFatalError( const char * format, ... )
 {
    va_list ap;
    char    msg[UFDB_MAX_URL_LENGTH];
-   char    logmsg[UFDB_MAX_URL_LENGTH];
+   char    logmsg[UFDB_MAX_URL_LENGTH+100];
 
    va_start( ap, format );
    vsnprintf( msg, UFDB_MAX_URL_LENGTH-48, format, ap );
@@ -264,31 +282,17 @@ void ufdbLogFatalError( char * format, ... )
    sprintf( logmsg, "*FATAL* %s  *****", msg );
    ufdbLog( logmsg );
 
-   UFDBglobalFatalError = 1;
+   ufdbGV.fatalError = 1;
 }
 
 
-static void writePidFile( void )
+static void TermHandler( int sig )
 {
-   FILE * fp;
+   if (sig) { ; }       // prevent compiler warning
 
-   (void) unlink( globalPidFile );
-   fp = fopen( globalPidFile, "w" );
-   if (fp == NULL)
-      ufdbLogError( "cannot write to PID file %s", globalPidFile );
-   else
-   {
-      fprintf( fp, "%d\n", globalPid );
-      fclose( fp );
-      if (UFDBglobalDebugHttpd  || UFDBglobalDebug)
-         ufdbLogMessage( "wrote my pid (%d) to %s", globalPid, globalPidFile );
-   }
-}
-
-
-void removePidFile( void )
-{
-   (void) unlink( globalPidFile );
+   ufdbLogMessage( "signal TERM received." );
+   removeHttpdPidFile();
+   exit( 0 );
 }
 
 
@@ -305,15 +309,16 @@ static void mySignalHandler( int sig )
       case SIGSEGV:  strcpy( sigDesc, "SEGV" );  break;
       default:       sprintf( sigDesc, "%d", sig );  break;
    }
-   ufdbLogError( "signal %s received. exiting...", sigDesc );
+   ufdbLogMessage( "signal %s received. exiting...", sigDesc );
 
-   removePidFile();
+   removeHttpdPidFile();
    exit( 0 );
 }
 
 
 static void USR1signalReceived( int sig )
 {
+   if (sig) { ; }       // prevent compileer warning
    ufdbLogMessage( "USR1 signal received for logfile rotation" );
    UFDBrotateLogfile();
    ufdbLogMessage( "USR1 signal received for logfile rotation" );
@@ -324,23 +329,36 @@ static void closeAllFiles( void )
 {
    int fd;
 
-   for (fd = 3; fd < 150; fd++)
+   /* when ufdbguardd called execv() to start ufdbhttpd, all file descriptors were still open. close them now */
+   close( 0 );
+   close( 1 );
+
+   for (fd = 3; fd < 2*UFDB_MAX_THREADS + 32; fd++)
    {
       close( fd );
    }
 }
 
 
+static void badSignalHandler( int signum )
+{
+   ufdbLogMessage( "badSignalHandler: received signal %d", signum );
+   ufdbExecutePstack( "ufdbhttpd received a bad signal" );
+   sleep( 1 );
+   exit( 2 );
+}
+
+
 int main( int argc, char ** argv )
 {
    int    ch;
 
-   strcpy( progname, "ufdbhttpd" );
-   globalPid = getpid();
-   globalErrorLog = NULL;
+   strcpy( ufdbGV.progname, "ufdbhttpd" );
+   ufdbGV.pid = getpid();
+   ufdbGlobalLogfile = NULL;
 
    closeAllFiles();
-   ufdbSetGlobalErrorLogFile();
+   ufdbSetGlobalErrorLogFile( NULL, NULL, 0 );
 
    while ((ch = getopt(argc, argv, "Ddhi:p:I:l:L:U:V")) > 0)
    {
@@ -349,8 +367,7 @@ int main( int argc, char ** argv )
       	 runAsDaemon = 0;
 	 break;
       case 'd':
-	 UFDBglobalDebug = 1;
-      	 runAsDaemon = 0;
+	 ufdbGV.debug++;
 	 break;
       case 'i':
 	 strcpy( interface, optarg );
@@ -362,19 +379,19 @@ int main( int argc, char ** argv )
 	 strcpy( imagesDirectory, optarg );
 	 break;
       case 'l':
-	 UFDBglobalLogDir = optarg;
+	 strcpy( ufdbGV.logDirectory, optarg );
 	 break;
       case 'L':				/* undocumented -L option for alternate PID file */
-	 globalPidFile = optarg;
+	 globalHttpdPidFile = optarg;
          break;
       case 'U':
 	 if (strlen(optarg) <= 31)
-	    strcpy( UFDBglobalUserName, optarg );
+	    strcpy( ufdbGV.userName, optarg );
 	 else
 	    ufdbLogFatalError( "username supplied with -U option is too long" );
          break;
       case 'V':
-         UFDBglobalDebugHttpd = 1;
+         ufdbGV.debugHttpd = 1;
 	 break;
       case '?':
       case 'h':
@@ -383,11 +400,11 @@ int main( int argc, char ** argv )
       }
    }
 
-   if (UFDBglobalLogDir == NULL ||
+   if (ufdbGV.logDirectory[0] == '\0' ||
        imagesDirectory[0] == '\0'  ||
        portnumber <= 0)
    {
-      ufdbLogFatalError( "%s started with incorrect parameters. aborting...", progname );
+      ufdbLogFatalError( "%s started with incorrect parameters. aborting...", ufdbGV.progname );
       usage();
       exit( 1 );
    }
@@ -399,31 +416,40 @@ int main( int argc, char ** argv )
       {
 	 exit( 0 );        
       }
-      if (setsid() < 0)
-         ufdbLogFatalError( "cannot create daemon session: %s", strerror(errno) );
+      /* ufdbhttpd is started by ufdbguardd and does not need a new session */
    }
-   globalPid = getpid();
+   ufdbGV.pid = getpid();
 
    ufdbSetSignalHandler( SIGPIPE, SIG_IGN );
    ufdbSetSignalHandler( SIGUSR1, USR1signalReceived );
-   ufdbSetSignalHandler( SIGTERM, mySignalHandler );
    ufdbSetSignalHandler( SIGHUP,  mySignalHandler );
    ufdbSetSignalHandler( SIGINT,  mySignalHandler );
-   
-   writePidFile();
-   atexit( removePidFile );
-   ufdbLogMessage( "%s " VERSION " started\ninterface=%s  port=%d  images=%s",
-                   progname, interface, portnumber, imagesDirectory );
-   if (UFDBglobalDebug)
-      ufdbLogMessage( "debug option is ON" );
-   if (UFDBglobalDebugHttpd)
-      ufdbLogMessage( "HTTP debug option is ON" );
-   if (UFDBglobalUserName[0] != '\0')
-      ufdbLogMessage( "dropping privileges to user %s", UFDBglobalUserName );
+   ufdbSetSignalHandler( SIGTERM, TermHandler );
 
-   ufdbSimulateHttpServer( interface, portnumber, UFDBglobalUserName, imagesDirectory, 0 );
+   ufdbSetSignalHandler( SIGBUS,  badSignalHandler );
+   ufdbSetSignalHandler( SIGILL,  badSignalHandler );
+   ufdbSetSignalHandler( SIGSEGV, badSignalHandler );
+   
+   ufdbSetGlobalErrorLogFile( NULL, NULL, 0 );
+
+   ufdbLogMessage( "%s " UFDB_VERSION " started\n"
+                   "interface=%s  port=%d  images=%s",
+                   ufdbGV.progname, interface, portnumber, imagesDirectory );
+   if (ufdbGV.debug)
+      ufdbLogMessage( "debug option is ON" );
+   if (ufdbGV.debugHttpd)
+      ufdbLogMessage( "HTTP debug option is ON" );
+   if (ufdbGV.userName[0] != '\0')
+      ufdbLogMessage( "dropping privileges to user %s", ufdbGV.userName );
+
+   ufdbSimulateHttpServer( interface, portnumber, ufdbGV.userName, imagesDirectory );
 
    exit( 0 );
    return 0;  /* make compiler happy */
 }
 
+
+#ifdef __cplusplus
+}
+#endif
+
